/*! \file TransientInput.h 
 *  \brief: header file for transientinput object
 */

#ifndef _TRANSIENTINPUT_H_
#define _TRANSIENTINPUT_H_

/*Headers:*/
/*{{{*/
#include "./Input.h"
class Gauss;
class Parameters;
class Gauss;
/*}}}*/

class TransientInput: public Input{

	public:
		int         enum_type;
		int         numtimesteps;
		Inputs     *inputs;
		IssmDouble *timesteps;
		Parameters *parameters;     //to find current time.

		/*TransientInput constructors, destructors: {{{*/
		TransientInput();
		TransientInput(int enum_type);
		TransientInput(int in_enum_type,IssmDouble* times,int N);
		~TransientInput();
		void AddTimeInput(Input* input,IssmDouble time);
		void AddTimeInput(Input* input);
		/*}}}*/
		/*Object virtual functions definitions:{{{*/
		void  Echo();
		void  DeepEcho();
		int   Id();
		int   ObjectEnum();
		Object* copy();
		/*}}}*/
		/*TransientInput management: {{{*/
		int    InstanceEnum();
		Input* SpawnTriaInput(int index1,int index2,int index3);
		Input* SpawnSegInput(int index1,int index2);
		Input* PointwiseDivide(Input* input_in){_error_("not implemented yet");};
		Input* PointwiseMin(Input* input_in){_error_("not implemented yet");};
		Input* PointwiseMax(Input* input_in){_error_("not implemented yet");};
		int  GetResultInterpolation(void);
		int  GetResultNumberOfNodes(void);
		void ResultToPatch(IssmDouble* values,int nodesperelement,int sid){_error_("not supported yet");};
		void Configure(Parameters* parameters);
		/*}}}*/
		/*numerics: {{{*/
		void GetInputValue(bool* pvalue){_error_("not implemented yet");};
		void GetInputValue(int* pvalue){_error_("not implemented yet");};
		void GetInputValue(IssmDouble* pvalue){_error_("not implemented yet");};
		void GetInputValue(IssmDouble* pvalue,Gauss* gauss);
		void GetInputValue(IssmDouble* pvalue,Gauss* gauss,IssmDouble time);
		void GetInputValue(IssmDouble* pvalue,Gauss* gauss ,int index){_error_("not implemented yet");};
		void GetInputDerivativeValue(IssmDouble* derivativevalues, IssmDouble* xyz_list,Gauss* gauss);
		void GetInputAverage(IssmDouble* pvalue);
		void GetInputAllTimeAverages(IssmDouble** pvalues,IssmDouble** ptimes, int* pnumtimes);
		void GetInputUpToCurrentTimeAverages(IssmDouble** pvalues, IssmDouble** ptimes, int* pnumtimes, IssmDouble currenttime);
		void ChangeEnum(int newenumtype);
		void SquareMin(IssmDouble* psquaremin,Parameters* parameters);
		void ConstrainMin(IssmDouble minimum){_error_("not implemented yet");};
		void Set(IssmDouble setvalue){_error_("Set not implemented yet");};
		void Scale(IssmDouble scale_factor){_error_("not implemented yet");};
		void AXPY(Input* xforcing,IssmDouble scalar){_error_("not implemented yet");};
		void Constrain(IssmDouble cm_min, IssmDouble cm_max){_error_("not implemented yet");};
		IssmDouble InfinityNorm(void);
		IssmDouble Max(void);
		IssmDouble MaxAbs(void);
		IssmDouble Min(void);
		IssmDouble MinAbs(void);
		void Extrude(int start);
		void VerticallyIntegrate(Input* thickness_forcing){_error_("not supported yet");};
		void GetVectorFromInputs(Vector<IssmDouble>* vector,int* doflist);
		void GetTimeValues(IssmDouble* values,IssmDouble time){_error_("not implemented yet");};
		Input* GetTimeInput(IssmDouble time);
		/*}}}*/

};
#endif  /* _TRANSIENTINPUT_H */
